@@ -196,14 +196,18 @@ namespace acommon {
   
     PosibErr<const KeyInfo *> keyinfo(ParmStr key) const;
 
-    KeyInfoEnumeration * possible_elements(bool include_extra = true) const;
-
+    KeyInfoEnumeration * possible_elements(bool include_extra = true,
+                                           bool include_modules = true) const;
+    
     StringPairEnumeration * elements() {return 0;} // FIXME
     
     String get_default(const KeyInfo * ki) const;
     PosibErr<String> get_default(ParmStr) const;
 
     PosibErr<String> retrieve(ParmStr key) const;
+
+    // will also retrive a list, with one value per line
+    PosibErr<String> retrieve_any(ParmStr key) const;
   
     bool have (ParmStr key) const;
 
@@ -258,8 +262,10 @@ namespace acommon {
     virtual ~KeyInfoEnumeration() {}
   };
 
-  static const int KEYINFO_MAY_CHANGE = 1 << 0;
-  static const int KEYINFO_UTF8       = 1 << 1;
+  static const unsigned KEYINFO_MAY_CHANGE = 1 << 0;
+  static const unsigned KEYINFO_UTF8       = 1 << 1;
+  static const unsigned KEYINFO_HIDDEN     = 1 << 2;
+  static const unsigned KEYINFO_COMMON     = 1 << 4;
   
   class AddableContainer;
   class StringList;
